/***********************************
*     PROJECT :MPM                 
*        FILE :MTO_PrnLabel.prg
*  Programmer :James Loughner
*  Created    :03/16/2009
*  
*  Label print class
*************************************/
 
#include "Gra.ch"
#include "Xbp.ch"
#include "Common.ch"
#include "Appevent.ch"
#include "Font.ch"
#include "dll.ch"
#include "inkey.ch" 
#include "Font.ch"
#include "XBPDEV.CH"

//  must include prnsay.ch to use @..PRNSAY commands
#include "PRNSAY.ch"

//#define TESTLABELS
// note: uses functions from the MTO_Printit.prg   
#ifdef TESTLABELS
PROCEDURE Test_Label
LOCAL I,oLabel,aTxt[4]
   oLabel := PrintLabel():New()
   oLabel:Rows := 10
   oLabel:Columns := 4
    
   IF !EMPTY(oLabel:Create())    //returns NIL on user abort
      FOR I = 1 TO 200
          aTxt := {"Label "+STR(I,3),;
                   "Address1",;
                   "Address2",;
                   "City, State, Zip";
                  }
          oLabel:PrintNext(aTxt)
      NEXT I
      oLabel:Destroy()
      MSGBOX("Done")
   ENDIF
RETURN
#endif


Class PrintLabel
   EXPORTED:
      METHOD Init,Create,Destroy,PrintNext
      VAR Rows,Columns,PageSize,Units,MaxLineLength,nLinesPer  
      VAR aCurLabel,cFontName,PointSize,lCenterPrint,PrintSeq,LabelX,LabelY 
  PROTECTED:
      VAR oFont,oPS,oPrinter,aLabel,LabelWidth,LabelHeight
ENDCLASS

METHOD PrintLabel:Init() 
   ::Rows          := 1
   ::Columns       := 1 
   ::Units         := GRA_PU_LOMETRIC // 0.1 mm
   ::aCurLabel     := {1,1}   // X,Y
   ::nLinesPer     := 4
   ::MaxLineLength := 40
   ::cFontName     := "Courier New"
   ::PointSize     := 12
   ::lCenterPrint  := .F. 
   ::PrintSeq      := "LR"  // LR left to right TB top to bottom  define sequence of print 
   ::LabelX        := 40   // set label left margin in units see above
   ::LabelY        := 40   // set label Top and bottom margins in units see above
RETURN Self

METHOD PrintLabel:Create() 
LOCAL aPrint,aTopRight,aPaperSize,X,Y,L
   aPrint := StartPrinter(::cFontName,::PointSize,NIL,NIL,.F.,SetAppWindow(),XBPPRN_ORIENT_PORTRAIT,NIL,.F.,::Units) 
   IF EMPTY(aPrint)
      ::Destroy()
      RETURN NIL   // to sign that print was aborted
   ENDIF  
   ::oPS      := aPrint[1]
   ::oFont    := aPrint[2]
   ::oPrinter := aPrint[3] 
   SetFont(::oFont)   // set default font
   SetReference(::oFont) // set font to define a grid on the paper
   SetoPS(::oPS)// set default presentation space

   GraSetFont(::oPS , ::oFont )
   ::aLabel   := Array(::Rows,::Columns,::nLinesPer)
   // calc and store start position for each label/line on page  
   aPaperSize  := ::oPrinter:paperSize()
   aTopRight   :={aPaperSize[1]-(aPaperSize[3]+(aPaperSize[1]-aPaperSize[5])),aPaperSize[2] - (aPaperSize[4]+(aPaperSize[2]-aPaperSize[6]))}
   ::LabelWidth  := INT(aTopRight[1]/::Columns)
   ::LabelHeight := INT(aTopRight[2]/::Rows) 
   LineSpace  := INT((::LabelHeight-2*::LabelY)/::nLinesPer)
   FOR Y = 1 TO ::Rows
      FOR X = 1 TO ::Columns 
         FOR L = 1 TO ::nLinesPer 
            ::aLabel[Y,X,L] := {(X-1)*::LabelWidth+::LabelX, aTopRight[2]-((Y-1)*::LabelHeight+L*LineSpace+::LabelY)} 
         NEXT L
      NEXT X
   NEXT Y  

RETURN Self

METHOD PrintLabel:PrintNext(aText)
LOCAL I
   // position to current label
   //Print text  
   FOR I = 1 TO LEN(aText)
      IF I > ::nLinesPer 
         EXIT
      ENDIF
      GraStringAt( ::oPS, ::aLabel[::aCurLabel[2],::aCurLabel[1],I], LEFT(aText[I],::MaxLineLength) )
   NEXT I 
   // set next to print
   IF ::PrintSeq = "LR" 
      ++::aCurLabel[1]
      IF ::aCurLabel[1]> ::Columns
         ::aCurLabel[1] := 1
         ++::aCurLabel[2]
         IF ::aCurLabel[2] > ::Rows
            ::aCurLabel[1] := 1
            ::aCurLabel[2] := 1  
            Printer_EJECT(::oPS)
         ENDIF   
      ENDIF   
   ELSEIF ::PrintSeq = "TB"
      ++::aCurLabel[2]
      IF ::aCurLabel[2]> ::Rows
         ::aCurLabel[2] := 1
         ++::aCurLabel[1]
         IF ::aCurLabel[1] > ::Columns
            ::aCurLabel[1] := 1
            ::aCurLabel[2] := 1  
            Printer_EJECT(::oPS)
         ENDIF   
      ENDIF   
   ENDIF
RETURN Self

METHOD PrintLabel:Destroy()
   IF !EMPTY(::oPs)
      End_Doc()
      EndPrinter(::oPS)
   ENDIF
RETURN Self

